#ifndef GRID_H          /* Avoid redefinitions                                */
#define GRID_H

#define MAXGRIDVALS    20 /* Max number of values stored for each grid point  */



/* The GRID structure contains a geometric mapping grid for mapping values
   from an input image to an output image.  A grid may map from 1 to
   MAXGRIDVALS values with the restriction that all of the values must be of
   the same data type.
   
   The mapping grids are in output space coordinates (line, sample).  Each grid
   contains 'nrows' rows and 'ncols' columns.  The field 'out_lines' is a list
   of the line numbers that make up the horizontal rows of the grid and
   'out_samps' is a list of the sample numbers that make up the vertical
   columns of the grid.  This structure also contains information needed to
   fill the output image's DDR and miscellaneous mapping grid parameters and
   statistics.

   EXAMPLE:

      An output image with 10 lines and 13 samples.  In the diagram the grid
      points are marked with the letters A through P.  Assume that each grid
      point has an x and y value associated with it.

      nrows     = 4
      ncols     = 4
      nvalues   = 2
      out_lines = 1 4 7 10
      out_samps = 1 5 9 13
      labels[0] = "x value"
      labels[1] = "y value"
      values[0] = Ax Bx Cx Dx Ex Fx Gx Hx Ix Jx Kx Lx Mx Nx Ox Px
      values[1] = Ay By Cy Dy Ey Fy Gy Hy Iy Jy Ky Ly My Ny Oy Py
      
      A---B---C---D
      |   |   |   |
      |   |   |   |
      E---F---G---H
      |   |   |   |
      |   |   |   |
      I---J---K---L
      |   |   |   |
      |   |   |   |
      M---N---O---P


   When storing a grid to a label services file the character fields grid.model
   and grid.units are stored in the first record, the double fields in the
   second record, the long fields in the third record, and then each of the
   individual elements of grid.labels and grid.values are paired up and stored
   in a record by themselves.  Along with the character data in the first
   record we also store the number of double and long values that are contained
   in records two and three, respectively.  This allows us to dynamically
   allocate the appropriate amount of memory before reading these records.

   Thus, the steps to read a grid file are:

      1. Read record one (fixed length)
      2. Allocate memory for record two
      3. Read record two
      4. Allocate memory for record three
      5. Read record three
      6. Allocate memory for each element of grid.values
      7. Read each element of grid.values

   Key      Character buffer             Data buffer
   -------  ---------------------------  ---------------------------------------
   RECORD1  "GEOGRID: model units"       ndoubles nlongs
   RECORD2  "GEOGRID: "                  All double fields in the grid
   RECORD3  "GEOGRID: "                  All long fields in the grid
   RECORD4  "labels[0]"                  grid->values[0]
   RECORD5  "labels[1]"                  grid->values[1]
      .         .                              .
      .         .                              .
==============================================================================*/
struct GRID
   {
   char gridname[256];     /* Host file name of the grid file                 */

   char labels[MAXGRIDVALS][CMLEN]; /* A character string describing each of
                              the values stored in grid.values.               */

   char model[15];         /* Type of model used to generate the geometric
                              mapping grid.  Standard values are "POLYNOMIAL",
                              "SPACECRAFT", and "FINITE ELEMENT".  Other values
                              are allowed, if need be.                        */

   char units[12];         /* Projection units.  This value corresponds to the
                              proj_units field in the DDR.                    */

   double act_grid_err[32];/* Actual fitting errors for each of the 16
                              reference points; each pair of values is given in
                              (line, sample) order, going left to right in each
                              row starting with the top row                   */

   double ave_fit_err[2];  /* Average of the absolute values of the horizontal
                              and vertical residual errors of the tie points
                              from the modeling process, if tie points were
                              used and model = "POLYNOMIAL."                  */

   double ave_grid_err[2]; /* Average of the absolute values of the horizontal
                              and vertical grid interpolation errors of the
                              sixteen reference points.                       */

   double *coeffs;         /* Pointer to transformation coefficients.  The
                              first ncoeff values are the Y transformation
                              coefficients, followed by ncoeff X tranformation
                              coefficients.                                   */

   double corners[8];      /* Projection coordinates of the resulting output
                              image's four corners.  This field corresponds to
                              the DDR's "upleft", "loleft", "upright", and
                              "loright" fields.                               */

   double grid_tol;        /* Tolerance used in reducing grid                 */

   double max_fit_err[2];  /* Maximum horizontal and vertical residual errors
                              of the tie points from the modeling process, if
                              tie points were used and model = "POLYNOMIAL".  */

   double max_grid_err[2]; /* Maximum horizontal and vertical grid interpolation
                              errors at an of 16 reference points.  Gridding
                              routines give an estimate of the overall quality
                              of the gridding process using a 4 x 4 matrix of
                              reference points located at the intersections of
                              four vertical and four horizontal lines in the
                              image region covered by the mapping grid.  The
                              lines are located 3/126, 43/126, 83/126, and
                              123/126 of the distance from one edge of the area
                              to the other.                                   */

   double pdist[2];        /* Projection distance per pixel in Y and in X.
                              This field corresponds to the "pdist_y" and
                              "pdist_x" field in the DDR.                     */

   double projprms[15];    /* Array of 15 projection coefficients as required
                              by the projection transformation package.  Refer
                              to the projection package documentation for a
                              description of each field for a given projection.
                              This field corresponds to the "proj_coef" field
                              in the DDR.                                     */

   double rms_fit_err;     /* RMS of the residual errors from the modeling
                              process, if tie points were used and model =
                              "POLYNIOMIAL."                                  */

   double rms_grid_err;    /* RMS of the residual errors of the sixteen
                              reference points.                               */

   long code;              /* Projection code for the output space image's DDR.
                              Values for this field are defined in the "proj.h"
                              include file.                                   */

   long datum;             /* Projection datum code.  It has the same valid
                              range and defaults as the "datum_code" field in
                              the DDR.                                        */

   long dtype;             /* The data type of all the values to be stored in
			      the grid.  Valid values are:

                                 EBYTE   = unsigned char
                                 EWORD   = int
                                 ELONG   = long
                                 EREAL   = float
                                 EDOUBLE = double
                                 ECHAR   = char                               */

   long fiterr_valid;      /* Valid field for fit error fields                */

   long griderr_valid;     /* Valid field for griding error fields            */

   long lines;             /* Number of lines in the output image             */

   long ncoeff;            /* Number of polynomial coefficients in X & Y.  This
                              field is zero if other than a polynomial model
                              was used to generate the mapping grid.          */

   long ncols;             /* Number of grid columns through the output image */

   long nrows;             /* Number of grid lines through the output image   */

   long nvalues;           /* Number of values to be contained in the grid.   */

   long *out_lines;        /* Pointer to the locations of the horizontal
                              coordinate (line number) on each grid row, going
                              from top to bottom.  The output grid cells are of
                              consistent size and are rectangular, perpendicular
                              to the line-sample axis of the output image.
                              Therefore, only intersections need be given.    */

   long *out_samps;        /* Pointer to the locations of the vertical
                              coordinate (pixel number) on each grid column,
                              going from left to right.  Again, only
                              intersections need be given.                    */

   long proj_valid;        /* Valid field for projection fields               */

   long samples;           /* Number of samples in the output image           */

   long satnum;            /* Number of satellite that acquired the data      */

   long zone;              /* Projection zone code for UTM or State Plane
                              projections.  It has the same valid ranges and
                              defaults as the "zone_code" field in the DDR.   */

   void *values[MAXGRIDVALS];/* Array of pointers to the values stored in the
			      grid.  These values are stored in row major
			      format for each grid point.                     */
   };

#endif
