#ifndef MODEL_H
#define MODEL_H

#include "vector.h"		/* Vector data defines			     */
#include "dropped.h"

/*
  Setup the model, orbital, and ephemeris information in the model structure
*/
#define HDR_SIZ        80	/* Size of a header record		     */
#define HDR_RECS       10	/* Number of image header records	     */
#define HDR_RECS_V2     7	/* Number of image header records	     */
#define ATT_RECS        4	/* Number of attitude correction records     */
#define CEOS_RECS      22	/* Total CEOS, HDR, and ATT records	     */
#define MIN_CEOS_RECS  21	/* Min number of CEOS records (using ver 1)  */
#define EPHEM_SIZ      70	/* Size of the ephemris data record	     */
#define N_ANGLES        3       /* Number of angles returned by the model    */
#define SATZEN          0       /* Index in angle array for satellite zenith */
#define SOLZEN          1       /* Index in angle array for solar zenith     */
#define RELAZ           2       /* Index in angle array for relative azimuth */
#define POLYCOEF 	5 	/* Poly. coefficients for altitude, attitude */ 

struct	MODEL
    {
    /*            START OF THE IMAGE INFORMATION			     */

    long   clock_err;		/* Satellite clock error (milliseconds)      */
    long   satnum;		/* Satellite number		             */
    long   dtype;		/* Data type  (1)LAC, (2)GAC, (3)HRPT        */
    char   station[4];		/* Station id				     */
    long   month;		/* Month of acquisition		             */
    long   day;			/* Day of acquisition                        */
    long   year;		/* Year of acquisition                       */
    long   jday;		/* Julian day of acquisition	             */
    long   shour;		/* Starting hour of acquisition              */
    long   smin;		/* Starting minute of acquisition            */
    long   ssec;		/* Starting second of acquisition            */
    long   sfrac;		/* Starting fractional seconds of acq        */
    long   ehour;		/* Ending hour of acquisition                */
    long   emin;		/* Ending minute of acquisition              */
    long   esec;		/* Ending second of acquisition              */
    long   efrac;		/* Ending fractional seconds of acq          */
    long   sorbit;		/* Orbit number at start of acquisition      */
    long   eorbit;		/* Orbit number at end of acquisition        */
    long   spass;		/* Direction of the pass at start            */
				/* (-1) Descending  (1) Ascending            */
    long   cpass;		/* Direction of the pass at center           */
				/* (-1) Descending  (1) Ascending            */
    long   epass;		/* Direction fo the pass at end	             */
				/* (-1) Descending  (1) Ascending            */
    long   nchans;		/* Number of channels in image               */
    long   chans[5];		/* Channels numbers in the image             */
    long   nlines;		/* Number of lines in image                  */
    long   nsamples;		/* Number of samples in image                */
    long   totmiss;		/* Total missed lines in image               */
    long   day_night;		/* Day (1) Night (-1) flag at center         */
    long   sat_view;		/* Satellite view (1) yes (0) data flipped   */
    double sunzen;		/* Sun zenith angle at center                */
    double nw[2];		/* Lat/Long at North West corner             */
    double nn[2];		/* Lat/Long at North Nadir point             */
    double ne[2];		/* Lat/Long at North East corner             */
    double cw[2];		/* Lat/Long at Center West point             */
    double cn[2];		/* Lat/Long at Center Nadir point            */
    double ce[2];		/* Lat/Long at Center East point             */
    double sw[2];		/* Lat/Long at South West corner             */
    double sn[2];		/* Lat/Long at South Nadir point             */
    double se[2];		/* Lat/Long at South East corner             */
    double eqcrs;		/* Equatorial crossing		             */

    /*            START OF THE EPHEMRIS INFORMATION			     */

    long   satid;		/* Satellite identifier (NOAA designator)    */
    long   orbit;		/* Ephemeris orbit number		     */
    long   epoch_jday;		/* Epoch julian date			     */
    long   epoch_year;		/* Epoch year				     */
    long   epoch_month;		/* Epoch month				     */
    long   epoch_day;		/* Epoch day				     */
    double mean_anomaly;	/* Mean anomaly (REVS)			     */
    double mean_motion;		/* Satellite mean motion  (RADS/HERG)	     */
    double decay;		/* Orbital decay (1.E-5 RADS/HERG^^2)	     */
    double ecc;			/* Eccentricity				     */
    double arg_per;		/* Argument of perigee (REVS)		     */
    double lon_anode;		/* Longitude of ascending node (REVS)	     */
    double inclin;		/* Inclination (REVS)			     */
    double smajor_axis;		/* Semi-major axis			     */
    double der_argper;		/* Derivative of argument of perigee	     */
    double der_anode;		/* Derivative of ascending node		     */
    double adot;		/* A dot?				     */
    double scene_ctime;		/* Time a center of pass           (seconds) */
    double mean_motion1;	/* Mean motion from center to start of pass  */
    double mean_motion2;	/* Mean motion from center to end of pass    */
    double sec_epoch;		/* Total seconds since epoch to first line   */
    double delta_time;		/* Delta time correction      (milliseconds) */
    double delta_alt;		/* Delta error in altitude          (meters) */
    double roll_coefs [POLYCOEF]; /* Roll correction coefficients    	     */
    double pitch_coefs[POLYCOEF]; /* Pitch correction coefficients   	     */
    double yaw_coefs  [POLYCOEF]; /* Yaw correction coefficients     	     */
    double alt_coefs  [POLYCOEF]; /* Altitude correction coefficients(meters)*/
    struct STATE satint;	/* Satellites initial position vectors       */
    };


/* COMMENTS DESCRIBING THE MODEL STRUCTURE 

time origin 
   Because LAS images are always "north up", AVHRR archive images are sometimes
   flipped upon conversion to LAS.  As a result, starting time corresponds to
   the first image line for descending passes and to the last image line for
   ascending passes.

polynomial coefficients 
   The coefficients roll_coefs[] pitch_coefs[] yaw_coefs[] alt_coefs[] give
   satellite position as a 4th degree polynomial function of time in seconds.
   Time=0 at scene center.  5 places in an array are required for a 4th degree
   polynomial hence "#define POLYCOEF 5".  Changing POLYCOEF will break source
   code that was written for only a 4th degree polynomial.

angular units
   roll_coefs[] pitch_coefs[] and yaw_coefs[] are stored on disk in degrees but
   converted immediately upon reading to radians.

eqcrs
   Equitorial crossing is used to describe the orbit to the inquiring user, it
   is not used for any calculations within the model.

alt_coefs 
   These coefficients describe radius (distance from earth center to satellite)
   and not alitude (distance from earth surface). 

EPOCH is 00:00:00.00 GMT
REV is 360 degrees (2*PI radians)

*/ 


/* Prototype functions to be called by any ADAPS support routine or module    */

void brouwer
(
    double *t,                  /* Total seconds past epoch             */
    struct MODEL *model,        /* Model and orbital information        */
    double *truea,              /* True anomaly past perigee            */
    double *rsat,               /* Distance of satellite from earth center*/
    double *thetas,             /* True anomaly past equator            */
    double *anlng,              /* Longitude of ascending node          */
    double *inclin,             /* Inclination angle                    */
    long *zdot,                 /* Ascending(1) / descending(-1) flag   */
    long *eorbit,               /* Orbit counter from equator           */
    long *porbit,               /* Orbit counter from perigee           */
    double *slat,               /* Geocenteric latitude of satellite    */
    double *slong               /* Longitude of satellite               */
);

int c_coords
(
    struct MODEL *model	/* IN/OUT - Model and orbital information	     */
);

void c_daynite
(
    long *jdate,	/* Julian date					*/
    long *year,		/* Year						*/
    double *tsec,	/* Time at point of interest			*/
    double *plat,	/* Latitude  of point				*/
    double *plong,	/* Longitude of point				*/
    double *sunzen,	/* Solar zenith angle				*/
    long *dayn		/* Day/Night indicator  Day(1) Night(-1)	*/
);

void c_forward
(
    double *line,	/* Input line number				     */
    double *sample,	/* Input sample number				     */
    double *tarelev,	/* Target elevation				     */
    struct MODEL *model,/* Model and orbital information		     */
    double *tarlatd,	/* Target latitude				     */
    double *tarlong,	/* Target longitude				     */
    double *view_angles	/* Calculated view angles			     */
);

void c_getorb
(
    double *tsec,	/* Total seconds at interest (GMT midnight)	*/
    struct MODEL *model,/* Model and orbital information		*/
    long *orbit,	/* Oribt number 				*/
    long *passdir,	/* Pass direction				*/
    double *eqxing	/* Equitorical crossing				*/
);

int c_initmod
(
    long *satnum,        /* Satellite number                     	     */
    long *year,          /* Year of the pass				     */
    long *month,         /* Month of the pass				     */
    long *day,           /* Day of the pass 				     */
    long *shour,         /* Start hour of pass                   	     */
    long *smin,          /* Start minute of pass                 	     */
    long *ssec,          /* Start second of pass                 	     */
    long *sfrac,         /* Start fractional seconds of pass     	     */
    long *dtype,         /* Type of data LAC(1), GAC(2) ,HRPT(3) 	     */
    long *nlines,	 /* Number of lines in the pass			     */
    struct MODEL *model	 /* Model and orbital information		     */
);

void c_inverse
(
    double *tarlatd,		/* Latitude of target			     */
    double *tarlong,		/* Longitude of target			     */
    double *tarelev,		/* Elevation of target			     */
    struct MODEL *model,	/* Model and orbital information	     */
    double *line,		/* Output line in scene			     */
    double *sample,	/* Output sample in scene			     */
    double *view_angles	/* Zenith and relative azimuth angles		     */
);

void c_ll2ls
(
    double *lat,		/* Latitude of point			*/
    double *lng,		/* Longitude of point			*/
    double *elev,		/* Elevation at the specified point	*/
    struct MODEL *model,	/* Model and orbital information	*/
    float *line,		/* Line in satellite scene		*/
    float *sample,		/* Sample in satellite scene		*/
    double view_angles[]   	/* Zenith and Azimuth angles		*/
);

void c_ls2ll
(
    float *line,		/* Line in satellite scene		*/
    float *sample,		/* Sample in satellite scene		*/
    double *elev,		/* Elevation at the specified point     */
    struct MODEL *model,	/* Model and orbital information	*/
    double *lat,		/* Latitude of point		        */
    double *lng,		/* Longitude of point		        */
    double view_angles[]   /* Zenith and relative azimuth angles, when  */
			   /* view_angles == NULL no angles are calculated */
);

int c_readephem
(
    long *satnum,		/* Satellite number			*/
    long *year,			/* Year of ephemeris looking for	*/
    long *month,		/* Month of ephemeris looking for	*/
    long *day,			/* Day of ephemeris looking for		*/
    char *eline			/* Ascii ephemris line			*/
);

int c_setephem
(
    char *eline,          /* ASCII ephemris line                */
    struct MODEL *model   /* Model and orbital information	*/
);

void c_setsat
(
    struct MODEL *model	/* Model and orbital information		     */
);

void c_sunpos
(
    long *jdate,		/* Julian date				*/
    long *year,			/* Year					*/
    double *tsec,		/* Total seconds from midnight GMT	*/
    double *sunlat,		/* Sun latitude location		*/
    double *sunlng		/* Sun longitude location		*/
);

int c_winddrp
(
    struct MODEL *model,	/* IN     - AVHRR model structure             */
    struct DROPPED *drop,	/* IN     - AVHRR dropped line structure      */
    struct DROPPED *window_drop,/* OUT    - windowed AVHRR dropped line struct*/
    long window[],		/* IN/OUT - window being applied to the model */
    long bound_flg		/* IN     - Bound flag:			      */
				/*	TRUE:  Allow the window to be bounded */
				/*	       by drop lines.		      */
				/*	FALSE: Don't allow the window to be   */
				/*	       bounded by drop lines.	      */
);

int c_windmod
(
    struct MODEL *model,	/* IN     - AVHRR model information           */
    struct DROPPED *drop,	/* IN     - AVHRR dropped line structure      */
    struct MODEL *window_model,	/* OUT    - AVHRR model of windowed info      */
    struct DROPPED *window_drop,/* OUT    - AVHRR windowed drop line structure*/
    long window[],		/* IN/OUT - Window for the AVHRR archive image*/
    long bands[],		/* IN     - Band spec for the output image    */
    long bound_flg              /* IN - Bound flag:			      */
                                /*      TRUE:  Allow the window to be bounded */
                                /*             by drop lines.                 */
                                /*      FALSE: Don't allow the window to be   */
                                /*             bounded by drop lines.         */
);


#endif

