
#ifndef C_QINCLUDE_H
#define C_QINCLUDE_H

/*******************************************************************************
MODULE:  c_queue

PURPOSE: Supply the necessary primitives for an ADT queue.

         1. Up to MAXQUEUES can be in use at one time.
         2. Each queue is completely independent of the other queues.
         3. Each queue can be specified as either ascending, descending, FIFO,
            or LIFO at the time of creation.
         4. Each queue can be specified as either allowing duplicates or not.
         5. The queues are doubly linked, circular lists with a dummy header
            node.
         6. Each queue is identified by a long variable that is returned by
            c_qcreate().
         7. All data stored in the queues should be dynamically allocated so
            that free() can be used to deallocate memory.
         8. The user must supply a comparison function that will accept two
            void pointers and will return a long value less than zero if the
            first object is less than the second; zero if the two objects are
            equal; and a long value greater than zero if the first object is
            greater than the second.
         9. If c_qprint() is to be called then the user must also supply a
            function that will accept a void pointer and print the object to
            standard output.

The following function is an example of a comparison function:
   
   long comp (ptr1, ptr2)
      void *ptr1;
      void *ptr2;
   {
   struct MYSTRUCT *p1;
   struct MYSTRUCT *p2;

   p1 = (struct MYSTRUCT *)ptr1;
   p2 = (struct MYSTRUCT *)ptr2;
   if (p1->value < p2->value)
      return(-1);
   else if (p1->value == p2->value)
      return(0);
   else
      return(1);
   }


Available functions:
   c_qcreate(order, duplicates)
   c_qdelete(q, position)
   c_qdestroy(q)
   c_qempty(q)
   c_qfind(q, objin, objout, compare, position)
   c_qfirst(q, ptr)
   c_qinsert(q, object, compare)
   c_qnext(q, ptr)
   c_qposition(q, position, ptr)
   c_qprev(q, ptr)
   c_qprint(q, print_obj)
   c_qsize(q)


Version  Programmer       Date   Modifications
-------  ---------------  -----  -----------------------------------------------
  1.0    J.Fenno          08/92  Development
  1.1    J.Fenno          10/92  Separated functions and created a library of
                                 queue functions.

*******************************************************************************/
#include <stdio.h>

#define MAXQUEUES 10
#define ASCEND     0
#define DESCEND    1
#define FIFO       2
#define LIFO       3

#define DUPS       0
#define NODUPS     1


/* The structure for a single queue node
----------------------------------------*/
struct QNODE
   {
   struct QNODE *prev;                   /* Previous element in the queue     */
   struct QNODE *next;                   /* Next element in the queue         */
   void   *obj;                          /* Void ptr to an object in the Q    */
   };


/* Prototype functions to be called by any ADAPS support routine or module
------------------------------------------------------------------------------*/
long c_qcreate
(
    char *order,        /* IN - String indicating the queue's order           */
    char *duplicates    /* IN - String indicating if duplicates are allowed   */
);

long c_qdelete
(
    long q,             /* IN - Number of a queue                             */
    long position       /* IN - Position in the queue to be deleted           */
);

long c_qdestroy
(
    long q              /* IN - Number of the queue to be destroyed           */
);

long c_qempty
(
    long q              /* IN - Number of the queue                           */
);

long c_qfind
(
    long q,             /* IN  - Number of the queue to search through        */
    void *objin,        /* IN  - Ptr to an object to find                     */
    void **objout,      /* OUT - Ptr to a ptr to the object that is found     */
    long (compare)(),	/* IN  - Function to compare two objects              */
    long *position      /* OUT - Position of the object in the queue          */
);

long c_qfirst
(
    long q,             /* IN  - Number of the queue to retrieve from         */
    void **ptr          /* OUT - Ptr to the first object in the queue         */
);

long c_qinsert
(
    long q,             /* IN - Number of the queue to insert into            */
    void *object,       /* IN - Ptr to an object to insert                    */
    long (compare)()    /* IN - Function to compare two objects               */
);

long c_qnext
(
    long q,             /* IN  - Number of the queue                          */
    void **ptr          /* OUT - Ptr to the next object in the queue          */
);

long c_qposition
(
    long q,             /* IN  - Number of the queue                          */
    long pos,           /* IN  - Position of interest                         */
    void **ptr          /* OUT - Ptr to the next object in the queue          */
);

long c_qprev
(
    long q,             /* IN  - Number of the queue                          */
    void **ptr          /* OUT - Ptr to the next object in the queue          */
);

long c_qprint
(
    long q,		    /* IN - Number of the queue to print              */
    void (print_obj)()      /* IN - Function to print an object in the queue  */
);

long c_qremove
(
    long q,             /* IN  - Number of a queue                            */
    long position,      /* IN  - Position in the queue to be deleted          */
    void **ptr          /* OUT - Ptr to the specified object in the queue     */
);

long c_qsize
(
    long q              /* IN - Number of the queue                           */
);

#endif

